import Foundation

enum GameType: Codable {
    
    enum GameClass: Int {
        case ruf = 1
        case bettel = 2
        case wenzGeier = 3
        case solo = 4
        
        var cost: Int {
            switch self {
            case .ruf: return 5
            case .bettel: return 15
            case .wenzGeier, .solo: return 20
            }
        }

        mutating func increase() {
            guard self != .solo else {
                return
            }
            self = .init(rawValue: rawValue + 1)!
        }
    }
    
    case rufEichel
    case rufBlatt
    case rufSchelln
    case hochzeit
    case bettel
    case wenz
    case geier
    case soloEichel
    case soloBlatt
    case soloHerz
    case soloSchelln
    
    var gameClass: GameClass {
        switch self {
        case .rufEichel, .rufBlatt, .rufSchelln, .hochzeit:
            return .ruf
        case .bettel:
            return .bettel
        case .wenz, .geier:
            return .wenzGeier
        case .soloEichel, .soloBlatt, .soloHerz, .soloSchelln:
            return .solo
        }
    }

    var isCall: Bool {
        switch self {
        case .rufEichel, .rufBlatt, .rufSchelln:
            return true
        default:
            return false
        }
    }

    var calledSuit: Card.Suit? {
        switch self {
        case .rufEichel:
            return .eichel
        case .rufBlatt:
            return .blatt
        case .rufSchelln:
            return .schelln
        default:
            return nil
        }
    }
    
    var isSingleGame: Bool {
        switch self {
        case .rufEichel, .rufBlatt, .rufSchelln, .hochzeit:
            return false
        default:
            return true
        }
    }
    
    var basicCost: Int {
        gameClass.cost
    }
    
    var sortingType: CardOrder.Type {
        switch self {
        case .wenz:
            return WenzCardOrder.self
        case .geier:
            return GeierCardOrder.self
        case .soloEichel:
            return SoloEichelCardOrder.self
        case .soloBlatt:
            return SoloBlattCardOrder.self
        case .soloSchelln:
            return SoloSchellnCardOrder.self
        default:
            return NormalCardOrder.self
        }
    }
}
